unit CountryList;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls;

// ========================================================================== //
//  TCountryList                                                              //
//                                                                            //
//  Simple component overriding Delphi's TComboBox                            //
//  Automaticaly receives list of contries using TAPI 2                       //
//  Adds following importent properties :                                     //
//    CountryName   -                                                         //
//  Following propeties are read/only and are suitable after setting          //
//  CoutryName to some real value (all these are of String type)              //
//    CountryCode   - contains Coutry Code of the country (e.g for Cuba it    //
//                    will be equal to 53                                     //
//    LongDistance  - Code that must be dialed when calling to other area     //
//    International - Code for making international call. Note that default   //
//                    number from TAPI 2 dosn't always coincide with accepted //
//    SameArea      - I don't know what this param means                      //
//
//  Author : Paul Krestol.                                                    //
//  Comments, suggestions, bugs etc please email to : michell_lam@hotmail.com //
//                                                                            //
//  Thank you for downloading one of my components                            //
// ========================================================================== //

// ========================================================================== //
// Interface to TAPI 2                                                        //
// Portions created by Microsoft are                                          //
// Copyright (C) 1995-1999 Microsoft Corporation.                             //
// All Rights Reserved.                                                       //
// The original file is: tapi.h                                               //
// ========================================================================== //

const
  TapiDll = 'tapi32.dll';

type

  PLineCountryList = ^TLineCountryList;
  TLineCountryList = packed record
    dwTotalSize,                                         // TAPI v1.4
    dwNeededSize,                                        // TAPI v1.4
    dwUsedSize,                                          // TAPI v1.4
    dwNumCountries,                                      // TAPI v1.4
    dwCountryListSize,                                   // TAPI v1.4
    dwCountryListOffset: DWORD;                          // TAPI v1.4
  end;

  PLineCountryEntry = ^TLineCountryEntry;
  TLineCountryEntry = packed record
    dwCountryID,                                         // TAPI v1.4
    dwCountryCode,                                       // TAPI v1.4
    dwNextCountryID,                                     // TAPI v1.4
    dwCountryNameSize,                                   // TAPI v1.4
    dwCountryNameOffset,                                 // TAPI v1.4
    dwSameAreaRuleSize,                                  // TAPI v1.4
    dwSameAreaRuleOffset,                                // TAPI v1.4
    dwLongDistanceRuleSize,                              // TAPI v1.4
    dwLongDistanceRuleOffset,                            // TAPI v1.4
    dwInternationalRuleSize,                             // TAPI v1.4
    dwInternationalRuleOffset: DWORD;                    // TAPI v1.4
  end;

  PCountryInfo = ^TCountryInfo;
  TCountryInfo = record
    CountryCode   : string;
    CountryName   : string;
    LongDistance  : string;
    International : string;
    SameArea      : string;
    NextID        : integer;
  end;

function lineGetCountry(dwCountryID, dwAPIVersion: DWORD;
  lpLineCountryList: PLineCountryList): Longint; stdcall; external TapiDll name 'lineGetCountryA';

// ========================================================================== //
//  TCountryList                                                              //
// ========================================================================== //

type

  TCountryList = class(TComboBox)
  private
    FList : TStringList;
    function IncPtr( const ptr : pointer; increment : integer ) : pointer;
    function RemoveABCs( str : string ) : string;
    function GetCountryCode: string;
    function GetInternational: string;
    function GetLongDistance: string;
    function GetSameArea: string;
    procedure SetCountryName(const Value: string);
    function GetCountryName: string;
  protected
    flgRefreshed : boolean;
    function  CountryInfo( CountryName : string ) : TCountryInfo;
    procedure EmptyList;
    procedure FillList;
    function  GetCountryAndCode( var CountryInfo : TCountryInfo ) : integer;
  public
    constructor Create(AOwner: TComponent); override;
    destructor  Destroy; override;
    procedure   Refresh;
    procedure   AfterConstruction; override;
  published
    property CountryName   : string read GetCountryName write SetCountryName;
    property CountryCode   : string read GetCountryCode;
    property LongDistance  : string read GetLongDistance;
    property International : string read GetInternational;
    property SameArea      : string read GetSameArea;
  end;

procedure Register;

implementation
{$R *.RES}


procedure Register;
begin
  RegisterComponents('Plus', [TCountryList]);
end;

{ TCountryList }

// ========================================================================== //
function TCountryList.CountryInfo(CountryName: string): TCountryInfo;
begin
  Result := PCountryInfo( FList.Objects[ Items.IndexOf( CountryName ) ] )^;
end;

// ========================================================================== //
constructor TCountryList.Create(AOwner: TComponent);
begin
  inherited;
  Style := csDropDownList;
  flgRefreshed := False;
  FList := TStringList.Create;
  FillList;
end;

// ========================================================================== //
destructor TCountryList.Destroy;
begin
  EmptyList;
  FList.Free;
  inherited;
end;

// ========================================================================== //
function TCountryList.IncPtr(const ptr: pointer;
  increment: integer): pointer;
begin
  Result := pointer( integer( ptr ) + increment );
end;

// ========================================================================== //
function TCountryList.RemoveABCs(str: string): string;
var
  i : integer;
begin
  Result := '';
  for i := 1 to length( str ) do
    case str[ i ] of
      '0'..'9' : Result := Result + str[ i ];
    end;
end;

// ========================================================================== //
function TCountryList.GetCountryAndCode( var CountryInfo : TCountryInfo ) : integer;
var
  lpLineCountryList : PLineCountryList;
  pCountryIn   : PLineCountryEntry;
  CountryIn    : TLineCountryEntry;
  pcharCountryName   : pChar;
  pcharLongDistance  : pChar;
  pcharInternational : pChar;
  pcharSameArea      : pChar;
begin
  GetMem( lpLineCountryList, 1024 );
  lpLineCountryList.dwTotalSize := 1024;
  Result := lineGetCountry( CountryInfo.NextID, $10003, lpLineCountryList );
  if Result <> 0 then Exit;
  pCountryIn := IncPtr( lpLineCountryList, lpLineCountryList^.dwCountryListOffset );
  CountryIn := pCountryIn^;

  pcharCountryName   := IncPtr( lpLineCountryList, pCountryIn^.dwCountryNameOffset );
  pcharLongDistance  := IncPtr( lpLineCountryList, pCountryIn^.dwLongDistanceRuleOffset );
  pcharInternational := IncPtr( lpLineCountryList, pCountryIn^.dwInternationalRuleOffset );
  pcharSameArea      := IncPtr( lpLineCountryList, pCountryIn^.dwSameAreaRuleOffset );

  CountryInfo.CountryName   := StrPas( pcharCountryName );
  CountryInfo.LongDistance  := RemoveABCs( StrPas( pcharLongDistance ) );
  CountryInfo.International := RemoveABCs( StrPas( pcharInternational ) );
  CountryInfo.SameArea      := RemoveABCs( StrPas( pcharSameArea ) );
  CountryInfo.CountryCode   := IntToStr( pCountryIn^.dwCountryCode );
  CountryInfo.NextID        := pCountryIn^.dwNextCountryID;
  FreeMem( lpLineCountryList, 1024 );
end;

// ========================================================================== //
procedure TCountryList.EmptyList;
var
  i : integer;
  ptrCountryInfo : PCountryInfo;
begin
  for i := 0 to FList.Count - 1 do
  begin
    ptrCountryInfo := PCountryInfo( FList.Objects[ i ] );
    Dispose( ptrCountryInfo );
  end;
end;

// ========================================================================== //
procedure TCountryList.FillList;
var
  ptrCountryInfo : PCountryInfo;
  CountryInfo : TCountryInfo;
begin
  CountryInfo.NextID := 1;
  while True do begin
    if GetCountryAndCode( CountryInfo ) = 0 then
    begin
      New( ptrCountryInfo );
      ptrCountryInfo^ := CountryInfo;
      FList.AddObject( CountryInfo.CountryName, TObject( ptrCountryInfo ) );
    end;
    if CountryInfo.NextID = 0 then Break;
  end;
  FList.Sort;
end;

// ========================================================================== //
procedure TCountryList.Refresh;
begin
  if flgRefreshed then Exit;
  flgRefreshed := True;
  Items.Assign( FList );
end;

// ========================================================================== //
function TCountryList.GetCountryCode: string;
begin
  case CountryName = '' of
    True : Result := '';
    False : Result := CountryInfo( CountryName ).CountryCode;
  end;
end;

// ========================================================================== //
function TCountryList.GetInternational: string;
begin
  case CountryName = '' of
    True : Result := '';
    False : Result := CountryInfo( CountryName ).International;
  end;
end;

// ========================================================================== //
function TCountryList.GetLongDistance: string;
begin
  case CountryName = '' of
    True : Result := '';
    False : Result := CountryInfo( CountryName ).LongDistance;
  end;
end;

// ========================================================================== //
function TCountryList.GetSameArea: string;
begin
  case CountryName = '' of
    True : Result := '';
    False : Result := CountryInfo( CountryName ).SameArea;
  end;
end;

// ========================================================================== //
procedure TCountryList.SetCountryName(const Value: string);
var
  NewIndex : integer;
begin
  Refresh;
  NewIndex := FList.IndexOf( Value );
  case NewIndex of
    -1 :
        Self.ItemIndex := -1;
    else
      Self.ItemIndex := NewIndex;
  end;
end;

// ========================================================================== //
function TCountryList.GetCountryName: string;
begin
  Refresh;
  Result := Self.Text;
end;

procedure TCountryList.AfterConstruction;
begin
  inherited;
  if not ( csDesigning in ComponentState ) then Refresh;
end;

end.


